<?php

/**
 * The admin-specific functionality of the plugin.
 *
 * @link       https://welaunch.io/plugins/wordpress-multilingual-multisite/
 * @since      1.0.0
 *
 * @package    WordPress_Multilingual_Multisite
 * @subpackage WordPress_Multilingual_Multisite/admin
 */

/**
 * The admin-specific functionality of the plugin.
 *
 * Defines the plugin name, version, and two examples hooks for how to
 * enqueue the admin-specific stylesheet and JavaScript.
 *
 * @package    WordPress_Multilingual_Multisite
 * @subpackage WordPress_Multilingual_Multisite/admin
 * @author     Daniel Barenkamp <support@welaunch.io>
 */
class WordPress_Multilingual_Multisite_Admin extends WordPress_Multilingual_Multisite {

	/**
	 * The ID of this plugin.
	 *
	 * @since    1.0.0
	 * @access   protected
	 * @var      string    $plugin_name    The ID of this plugin.
	 */
	protected $plugin_name;

	/**
	 * The version of this plugin.
	 *
	 * @since    1.0.0
	 * @access   protected
	 * @var      string    $version    The current version of this plugin.
	 */
	protected $version;

	/**
	 * Initialize the class and set its properties.
	 *
	 * @since    1.0.0
	 * @param      string    $plugin_name       The name of this plugin.
	 * @param      string    $version    The version of this plugin.
	 */
	public function __construct( $plugin_name, $version, $current_blog_id, $translation_tables ) 
	{
		$this->plugin_name = $plugin_name;
		$this->version = $version;
		$this->current_blog_id = $current_blog_id;
		$this->translation_tables = $translation_tables;
	}

    /**
     * Enqueue Admin Styles
     * @author Daniel Barenkamp
     * @version 1.0.0
     * @since   1.0.0
     * @link    https://welaunch.io/plugins/wordpress-multilingual-multisite/
     * @return  boolean
     */
    public function enqueue_styles()
    {
        wp_enqueue_style($this->plugin_name.'-admin', plugin_dir_url( dirname( __FILE__ ) ).'assets/css/wordpress-multilingual-multisite-admin.css', array(), $this->version, 'all');
        wp_enqueue_style('flag-icon', plugin_dir_url( dirname( __FILE__ ) ).'assets/css/flag-icon-css/css/flag-icon.min.css', array(), '3.1.0', 'all');
        wp_enqueue_style('font-awesome', 'https://cdnjs.cloudflare.com/ajax/libs/font-awesome/5.14.0/css/all.min.css', array(), '5.14.0', 'all');
        wp_enqueue_style('select2', 'https://cdnjs.cloudflare.com/ajax/libs/select2/4.0.13/css/select2.min.css', array(), '4.0.13');
    }

    /**
     * Enqueue Admin Scripts
     * @author Daniel Barenkamp
     * @version 1.0.0
     * @since   1.0.0
     * @link    https://welaunch.io/plugins/wordpress-multilingual-multisite/
     * @return  boolean
     */
    public function enqueue_scripts()
    {
    	global $pagenow;
        wp_enqueue_script($this->plugin_name.'-admin', plugin_dir_url( dirname( __FILE__ ) ).'assets/js/wordpress-multilingual-multisite-admin.js', array('jquery'), $this->version);
		if (( $pagenow == 'post.php' ) || ($pagenow == 'post-new.php')) {
		    wp_enqueue_script('select2', 'https://cdnjs.cloudflare.com/ajax/libs/select2/4.0.13/js/select2.min.js', array('jquery'), '4.0.13');
		}
    }

	public function load_redux()
	{
        if(!is_admin() || !current_user_can('manage_network')){
            return false;
        }

	    // Load the theme/plugin options
	    if ( file_exists( plugin_dir_path( dirname( __FILE__ ) ) . 'config/options-init.php' ) ) {
	        require_once plugin_dir_path( dirname( __FILE__ ) ) . 'config/options-init.php';
	    }
	}

	public function init() 
	{
		global $wordpress_multilingual_multisite;

        if(!is_admin() || !current_user_can('manage_network')){
            $wordpress_multilingual_multisite = get_network_option(1, 'wordpress_multilingual_multisite');
        } 

		$this->options = $wordpress_multilingual_multisite;
	}

	public function set_transients() 
	{
		global $blogs_data;

		$check = get_site_option( 'wordpress_multilingual_blogs_data');
		if($check) {
			$blogs_data = $check;
			return false;
		}

		$sites = get_sites();
		if(empty($sites)) {
			return false;
		}

		$blogs_data = array();
		$excludedSites = $this->get_option('excludedSites');

		foreach ($sites as $site) {

			$locale = "";
			$language = "";
			$country = "";

			$site_url = get_blog_option($site->blog_id, 'siteurl');
			// if($site->blog_id != 1) {
				$site_url = rtrim($site_url, '/') . '/';
			// }
			$admin_url = get_admin_url($site->blog_id);

			$data = array(
				'site_url' => $site_url,
				'admin_url' => $admin_url,
				
				'blog_id' => (int) $site->blog_id,
				'locale' => get_blog_option($site->blog_id, 'we_locale'),
				'language' => get_blog_option($site->blog_id, 'we_language'),
				'alternate' => get_blog_option($site->blog_id, 'we_alternate'),
				'country' => get_blog_option($site->blog_id, 'we_country'),
				'name' => get_blog_option($site->blog_id, 'blogname'),
				'country_selector_name' => get_blog_option($site->blog_id, 'we_country_selector_name'),
				'country_selector_flag_url' => get_blog_option($site->blog_id, 'we_country_selector_flag_url'),
			);

			$blogs_data[$site->blog_id] = $data;
		}


		$this->saving_options = true;
		update_site_option( 'wordpress_multilingual_blogs_data', $blogs_data );
		$this->saving_options = false;
	}

	public function add_admin_bar_menu()
	{
		global $wp_admin_bar, $blogs_data;

		if(!is_admin()) {
			return;
		}

		$adminLink = 'https://' . $_SERVER['HTTP_HOST'] . $_SERVER['REQUEST_URI'];
		$adminLinkArgs = str_replace($blogs_data[$this->current_blog_id]['admin_url'], '', $adminLink);
		$menu_id = 'wordpress-multilingual-multisite-admin-bar';

		$wp_admin_bar->add_menu( array(
			'id' => $menu_id,
			'title' => '<span class="flag-icon flag-icon-' . strtolower($blogs_data[$this->current_blog_id]['country']) . '"></span>',
			'href' => $blogs_data[$this->current_blog_id]['site_url']),
		);

		// Posts
		if(isset($_GET['post'])) {

			$originalPostId = intval($_GET['post']);
			$this->translation_tables->set_table('posts_translations');
			$translations_exists = $this->translation_tables->get_translated_items_by_item_id($this->current_blog_id, $originalPostId);

			foreach ($blogs_data as $blog_id => $blog_data) {
				if($blog_id == $this->current_blog_id) {
					continue;
				}

				if(isset($translations_exists[$blog_id])) {
					$wp_admin_bar->add_menu(array(
						'parent' => $menu_id,
						'title' => '<span class="flag-icon flag-icon-' . strtolower($blog_data['country']) . '"></span> ' . $blog_data['name'],
						'id' => 'wordpress-multilingual-multisite-admin-bar-' . $blog_id,
						'href' => $blog_data['admin_url'] . 'post.php?action=edit&post=' . $translations_exists[$blog_id], 
					));
				}
			}


		// Tags
		} elseif(isset($_GET['tag_ID'])) {

			$originalTagId = intval($_GET['tag_ID']);
			$originalTaxonomy = esc_attr($_GET['taxonomy']);
			$this->translation_tables->set_table('terms_translations');
			$translations_exists = $this->translation_tables->get_translated_items_by_item_id($this->current_blog_id, $originalTagId);

			foreach ($blogs_data as $blog_id => $blog_data) {
				if($blog_id == $this->current_blog_id) {
					continue;
				}

				if(isset($translations_exists[$blog_id])) {
					$wp_admin_bar->add_menu(array(
						'parent' => $menu_id,
						'title' => '<span class="flag-icon flag-icon-' . strtolower($blog_data['country']) . '"></span> ' . $blog_data['name'],
						'id' => 'wordpress-multilingual-multisite-admin-bar-' . $blog_id,
						'href' => $blog_data['admin_url'] . 'term.php?tag_ID=' . $translations_exists[$blog_id] . '&taxonomy=' . $originalTaxonomy, 
					));
				}
			}
	
		// Settings Menus etc
		} else {

			foreach($blogs_data as $blog_id => $blog_data) {
				if($blog_id == $this->current_blog_id) {
					continue;
				}

				$wp_admin_bar->add_menu(array(
					'parent' => $menu_id,
					'title' => '<span class="flag-icon flag-icon-' . strtolower($blog_data['country']) . '"></span> ' . $blog_data['name'],
					'id' => 'wordpress-multilingual-multisite-admin-bar-' . $blog_id,
					'href' => $blog_data['admin_url'] . $adminLinkArgs, 
				));
			}
		}

	}

	public function delete_transient($option)
	{
		global $wp;
		if (strpos($_SERVER['REQUEST_URI'], 'options-general.php') !== false || strpos($_SERVER['REQUEST_URI'], 'options.php') !== false || strpos($_SERVER['REQUEST_URI'], 'wordpress_multilingual_multisite_options') !== false) {
			update_site_option( 'wordpress_multilingual_blogs_data', '');
		}
	}
}