<?php

/**
 * The admin-specific functionality of the plugin.
 *
 * @link       https://welaunch.io/plugins/wordpress-multilingual-multisite/
 * @since      1.0.0
 *
 * @package    WordPress_Multilingual_Multisite
 * @subpackage WordPress_Multilingual_Multisite/admin
 */

/**
 * The admin-specific functionality of the plugin.
 *
 * Defines the plugin name, version, and two examples hooks for how to
 * enqueue the admin-specific stylesheet and JavaScript.
 *
 * @package    WordPress_Multilingual_Multisite
 * @subpackage WordPress_Multilingual_Multisite/admin
 * @author     Daniel Barenkamp <support@welaunch.io>
 */
class WordPress_Multilingual_Multisite_Blog_Copy extends WordPress_Multilingual_Multisite {

	protected $plugin_name;
	protected $version;

	/**
	 * Initialize the class and set its properties.
	 *
	 * @since    1.0.0
	 * @param      string    $plugin_name       The name of this plugin.
	 * @param      string    $version    The version of this plugin.
	 */
	public function __construct( $plugin_name, $version, $network_query, $translation_tables, $current_blog_id)
 	{
		$this->plugin_name = $plugin_name;
		$this->version = $version;
        $this->network_query = $network_query;
        
        $this->translation_tables = $translation_tables;

        $this->prefix = 'wordpress_multilingual_multisite_';
        $this->current_blog_id = $current_blog_id;

	}

	public function init() 
	{
		global $wordpress_multilingual_multisite;
		$this->options = $wordpress_multilingual_multisite;

		if (!$this->get_option('enable')) {
			return FALSE;
		}
	}

	public function add_menu()
	{
		add_submenu_page( 'sites.php', 'Copy Blog', 'Copy Blog', 'manage_sites', 'blog-copy', array( $this, 'admin_page' ) );		
	}

	public function add_site_action($actions, $blog_id )
	{
		if( is_main_site( $blog_id ) ) {
			return $actions;
		}
			
		$url = add_query_arg( array(
			'page' => 'blog-copy',
			'blog' => $blog_id
		), network_admin_url( 'sites.php' ) );
		$nonce_string = sprintf( '%s-%s', 'wordpress-multilingual-multisite', $blog_id );
		$actions['wordpress-multilingual-multisite'] = '<a href="' . esc_url( wp_nonce_url( $url, $nonce_string ) ) . '">' . esc_html__( 'Copy', 'wordpress-multilingual-multisite' ) . '</a>';

		return $actions;
	}


	public function admin_page() {
		global $wpdb, $current_site;

		if( !current_user_can( 'manage_sites' ) )
			wp_die( esc_html__( "Sorry, you don't have permissions to use this page.", 'wordpress-multilingual-multisite' ) );

		$from_blog = false;
		$copy_id = 0;
		$nonce_string = sprintf( '%s-%s', 'wordpress-multilingual-multisite', $copy_id );
		if( isset($_GET['blog']) && wp_verify_nonce( $_GET['_wpnonce'], $nonce_string ) ) {
			$copy_id = (int)$_GET['blog'];
			$from_blog = get_blog_details( $copy_id );
			if( $from_blog->site_id != $current_site->id ) {
				$from_blog = false;
			}
		}
		$from_blog_id = ( isset( $_POST['source_blog'] ) ) ? (int) $_POST['source_blog'] : -1;

		if( isset($_POST[ 'action' ]) && $_POST[ 'action' ] == 'wordpress-multilingual-multisite' ) {
			check_admin_referer( 'wordpress-multilingual-multisite' );
			$blog = $_POST['blog'];
			$domain = sanitize_user( str_replace( '/', '', $blog[ 'domain' ] ) );
			$title = $blog[ 'title' ];

			$copy_files = (isset($_POST['copy_files']) && $_POST['copy_files'] == '1') ? true : false;

			if ( !$from_blog_id ) {
				$msg = esc_html__( 'Please select a source blog.', 'wordpress-multilingual-multisite' );
			} elseif ( empty( $domain ) ) {
				$msg = esc_html__( 'Please enter a "New Blog Address".', 'wordpress-multilingual-multisite' );
			} elseif ( empty( $title ) ) {
				$msg = esc_html__( 'Please enter a "New Blog Title".', 'wordpress-multilingual-multisite' );
			} else {
				$msg = $this->copy_blog( $domain, $title, $from_blog_id, $copy_files );
			}
		} else {
			$copy_files = true; // set the default for first page load
		} ?>
		<div class='wrap'><h2><?php echo 'Copy Blog'; ?></h2><?php

		if( isset( $msg ) ) { ?>
		<div id="message" class="updated fade"><p><strong><?php echo wp_kses_post($msg); ?>
		</strong></p></div><?php
		}
		if( !$from_blog ) {
			$query = "SELECT b.blog_id, CONCAT(b.domain, b.path) as domain_path FROM {$wpdb->blogs} b " .
				"WHERE b.site_id = {$current_site->id} ORDER BY domain_path ASC LIMIT 10000";

			$blogs = $wpdb->get_results( $query );
		}
		if( $from_blog || $blogs ) { ?>
		<div class="wrap">
		<h3><?php esc_html_e( 'Blog Copy Settings', 'wordpress-multilingual-multisite' ); ?></h3>
		<p>Before you copy a site at first time, make sure you have clicked on INIT Translations button in our plugin settings > general.</p>
			<form method="POST">
				<input type="hidden" name="action" value="<?php echo 'wordpress-multilingual-multisite'; ?>" />
					<table class="form-table">

						<?php if( $from_blog ) { ?>
						<tr>
							<th scope='row'><?php esc_html_e( 'Source Blog to Copy', 'wordpress-multilingual-multisite' ); ?></th>
							<td><strong><?php printf( '<a href="%s" target="_blank">%s</a>', $from_blog->siteurl, $from_blog->blogname ); ?></strong>
							<input type="hidden" name="source_blog" value="<?php echo esc_attr($copy_id); ?>" />
							</td>
						</tr>
						<?php } else { ?>
						<tr class="form-required">
							<th scope='row'><?php esc_html_e( 'Choose Source Blog to Copy', 'wordpress-multilingual-multisite' ); ?></th>
							<td>
								<select name="source_blog">
								<?php foreach( $blogs as $blog ) {
										$selectedBlog = isset($_GET['blog']) ? $_GET['blog'] : '';
								 ?>
									<option value="<?php echo esc_attr($blog->blog_id); ?>" <?php selected( $blog->blog_id, $selectedBlog ); ?>><?php echo substr($blog->domain_path, 0, -1); ?></option>
								<?php } ?>
								</select>
							</td>
						</tr>
						<?php } ?>

						<tr class="form-required">
							<th scope='row'><?php esc_html_e( 'New Blog Address', 'wordpress-multilingual-multisite' ); ?></th>
							<td>
							<?php if( is_subdomain_install() ) { ?>
								<input name="blog[domain]" type="text" title="<?php esc_html_e( 'Subdomain', 'wordpress-multilingual-multisite' ); ?>" class="regular-text"/>.<?php echo wp_kses_post($current_site->domain);?>
							<?php } else {
								echo wp_kses_post($current_site->domain) . $current_site->path ?><input name="blog[domain]" type="text" title="<?php esc_html_e( 'Domain', 'wordpress-multilingual-multisite' ); ?>" class="regular-text"/>
							<?php } ?>
							</td>
						</tr>

						<tr class="form-required">
							<th scope='row'><?php esc_html_e( 'New Blog Title', 'wordpress-multilingual-multisite' ); ?></th>
							<td><input name="blog[title]" type="text" title="<?php esc_html_e( 'Title', 'wordpress-multilingual-multisite' ); ?>" class="regular-text"/></td>
						</tr>

						<tr class="form-required">
							<th scope='row'><?php esc_html_e( 'Copy Files?', 'wordpress-multilingual-multisite' ); ?></th>
							<td><input type="checkbox" name="copy_files" value="1" <?php checked( $copy_files ); ?>/></td>
						</tr>

					</table>
					<?php wp_nonce_field( 'wordpress-multilingual-multisite' ); ?>
				<p class="submit"><input class='button' type='submit' value='<?php esc_html_e( 'Copy Now', 'wordpress-multilingual-multisite' ); ?>' /></p>
			</form></div>
		<?php } else { ?>
			<div class="wrap">
				<h3><?php esc_html_e( 'Oops!', 'wordpress-multilingual-multisite' ); ?></h3>
				<p><?php
				printf( esc_html__( 'This plugin only works on subblogs. To use this you\'ll need to <a href="%s">create at least one subblog</a>.', 'wordpress-multilingual-multisite' ), network_admin_url( 'site-new.php' ) );
				?></p>
			</div>
		<?php }
	}

	/**
	 * Copy the blog
	 *
	 * @param string $domain url of the new blog
	 * @param string $title title of the new blog
	 * @param int $from_blog_id ID of the blog being copied from.
	 * @param bool $copy_files true if files should be copied
	 * @return string status message
	 */
	public function copy_blog($domain, $title, $from_blog_id = 0, $copy_files = true) {
		global $wpdb, $current_site, $base, $wordpress_multilingual_multisite_copy_running;

		$wordpress_multilingual_multisite_copy_running = true;

		$email = get_blog_option( $from_blog_id, 'admin_email' );
		$user_id = email_exists( sanitize_email( $email ) );
		if( !$user_id ) {
			// Use current user instead
			$user_id = get_current_user_id();
		}
		// The user id of the user that will become the blog admin of the new blog.
		$user_id = apply_filters('copy_blog_user_id', $user_id, $from_blog_id);

		if( is_subdomain_install() ) {
			$newdomain = $domain.".".$current_site->domain;
			$path = $base;
		} else {
			$newdomain = $current_site->domain;
			$path = trailingslashit( $base ) . trailingslashit( $domain );
		}

		// The new domain that will be created for the destination blog.
		$newdomain = apply_filters('copy_blog_domain', $newdomain, $domain);

		// The new path that will be created for the destination blog.
		$path = apply_filters('copy_blog_path', $path, $domain);

		$wpdb->hide_errors();
		$to_blog_id = wpmu_create_blog( $newdomain, $path, $title, $user_id , array( "public" => 1 ), $current_site->id );
		$wpdb->show_errors();

		if( !is_wp_error( $to_blog_id ) ) {
			$dashboard_blog = get_dashboard_blog();
			if( !is_super_admin() && get_user_option( 'primary_blog', $user_id ) == $dashboard_blog->blog_id )
				update_user_option( $user_id, 'primary_blog', $to_blog_id, true );

			// now copy
			if( $from_blog_id ) {

				$msg = '';

				$this->copy_blog_data( $from_blog_id, $to_blog_id );
				if ($copy_files) {

					if($from_blog_id == 1) {
						$msg .= 'When using blog id 1 to copy files you have to manually move all files from wp-content/uploads/ to the corresponsind /sites/ folder.';
					}

					$this->copy_blog_files( $from_blog_id, $to_blog_id );
					$this->replace_content_urls( $from_blog_id, $to_blog_id );

				}

				$q1 = 'UPDATE `' . $wpdb->prefix . 'terms_translations` SET `' . $to_blog_id . '`= `' . $from_blog_id . '`';				
				$wpdb->query( $q1 );


				$q2 = 'UPDATE `' . $wpdb->prefix . 'posts_translations` SET `' . $to_blog_id . '`= `' . $from_blog_id . '`';				
				$wpdb->query( $q2 );

				$msg .= sprintf(esc_html__( 'Copied: %s!', 'wordpress-multilingual-multisite' ), '<a href="http://'.$newdomain.'" target="_blank">'.$title.'</a>');
				do_action( 'log', esc_html__( 'Copy Complete!', 'wordpress-multilingual-multisite' ), 'wordpress-multilingual-multisite', $msg );
				do_action( 'copy_blog_complete', $from_blog_id, $to_blog_id );
			}
		} else {
			$msg = $to_blog_id->get_error_message();
		}

		delete_site_transient( 'wordpress_multilingual_multisite_blogs_data');

		$wordpress_multilingual_multisite_copy_running = false;

		return $msg;
	}

	/**
	 * Copy blog data from one blog to another
	 *
	 * @param int $from_blog_id ID of the blog being copied from.
	 * @param int $to_blog_id ID of the blog being copied to.
	 */
	private function copy_blog_data( $from_blog_id, $to_blog_id ) {
		global $wpdb, $wp_version;
		if( $from_blog_id ) {
			$from_blog_prefix = $this->get_blog_prefix( $from_blog_id );
			$to_blog_prefix = $this->get_blog_prefix( $to_blog_id );
			$from_blog_prefix_length = strlen($from_blog_prefix);
			$to_blog_prefix_length = strlen($to_blog_prefix);
			$from_blog_escaped_prefix = str_replace( '_', '\_', $from_blog_prefix );

			// Grab key options from new blog.
			$saved_options = array(
				'siteurl'=>'',
				'home'=>'',
				'upload_path'=>'',
				'fileupload_url'=>'',
				'upload_url_path'=>'',
				'admin_email'=>'',
				'blogname'=>''
			);
			// Options that should be preserved in the new blog.
			$saved_options = apply_filters('copy_blog_data_saved_options', $saved_options);
			foreach($saved_options as $option_name => $option_value) {
				$saved_options[$option_name] = get_blog_option( $to_blog_id, $option_name );
			}

			// Copy over ALL the tables.
			$query = $wpdb->prepare('SHOW TABLES LIKE %s',$from_blog_escaped_prefix.'%');
			do_action( 'log', $query, 'wordpress-multilingual-multisite');
			$old_tables = $wpdb->get_col($query);
			if($from_blog_id == 1) {
				$removeTables = array(
					'Jur38_blog_versions',
					'Jur38_blogmeta',
					'Jur38_blogs',
					'Jur38_domain_mapping',
					'Jur38_domain_mapping_logins',
					'Jur38_signups',
					'Jur38_site',
					'Jur38_sitemeta',
					'Jur38_usermeta',
					'Jur38_users',
				);
				foreach ($old_tables as $old_tableKey => $old_tableValue) {
					$rem = str_replace($from_blog_prefix, '', $old_tableValue);
					if(preg_match('/\\d/', $rem) > 0) {
						unset($old_tables[$old_tableKey]);
					}
					if(in_array($old_tableValue, $removeTables)) {
						unset($old_tables[$old_tableKey]);	
					}
				}
			}

			foreach ($old_tables as $k => $table) {
				$raw_table_name = substr( $table, $from_blog_prefix_length );
				$newtable = $to_blog_prefix . $raw_table_name;

				$query = "DROP TABLE IF EXISTS {$newtable}";
				do_action( 'log', $query, 'wordpress-multilingual-multisite');
				$wpdb->get_results($query);

				$query = "CREATE TABLE IF NOT EXISTS {$newtable} LIKE {$table}";
				do_action( 'log', $query, 'wordpress-multilingual-multisite');
				$wpdb->get_results($query);

				$query = "INSERT {$newtable} SELECT * FROM {$table}";
				do_action( 'log', $query, 'wordpress-multilingual-multisite');
				$wpdb->get_results($query);
			}

			switch_to_blog( $to_blog_id );

			// caches will be incorrect after direct DB copies
			wp_cache_delete( 'notoptions', 'options' );
			wp_cache_delete( 'alloptions', 'options' );

			// apply key options from new blog.
			foreach( $saved_options as $option_name => $option_value ) {
				update_option( $option_name, $option_value );
			}

			/// fix all options with the wrong prefix...
			$query = $wpdb->prepare("SELECT * FROM {$wpdb->options} WHERE option_name LIKE %s", $from_blog_escaped_prefix.'%' );
			$options = $wpdb->get_results( $query );
			do_action( 'log', $query, 'wordpress-multilingual-multisite', count($options).' results found.');
			if( $options ) {
				foreach( $options as $option ) {
					$raw_option_name = substr($option->option_name,$from_blog_prefix_length);
					$wpdb->update( $wpdb->options, array( 'option_name' => $to_blog_prefix . $raw_option_name ), array( 'option_id' => $option->option_id ) );
				}

				// caches will be incorrect after direct DB copies
				wp_cache_delete( 'notoptions', 'options' );
				wp_cache_delete( 'alloptions', 'options' );
			}

			// Fix GUIDs on copied posts
			$this->replace_guid_urls( $from_blog_id, $to_blog_id );

			restore_current_blog();
		}
	}

	/**
	 * Copy files from one blog to another.
	 *
	 * @param int $from_blog_id ID of the blog being copied from.
	 * @param int $to_blog_id ID of the blog being copied to.
	 */
	private function copy_blog_files( $from_blog_id, $to_blog_id ) {
		set_time_limit( 2400 ); // 60 seconds x 10 minutes
		@ini_set('memory_limit','2048M');

		// Path to source blog files.
		switch_to_blog($from_blog_id);
		$dir_info = wp_upload_dir();
		
		$from = str_replace(' ', "\\ ", trailingslashit($dir_info['basedir']).'*'); // * necessary with GNU cp, doesn't hurt anything with BSD cp
		restore_current_blog();
		$from = apply_filters('copy_blog_files_from', $from, $from_blog_id);

		// Path to destination blog files.
		switch_to_blog($to_blog_id);
		$dir_info = wp_upload_dir();
		
		$to = str_replace(' ', "\\ ", trailingslashit($dir_info['basedir']));

		$exploded = explode('/', $dir_info['basedir']);
		$explodedEnd = end($exploded);
		if($explodedEnd != $to_blog_id) {
			$to = $to . 'sites/' . $to_blog_id . '/';
		}

		restore_current_blog();
		$to = apply_filters('copy_blog_files_to', $to, $to_blog_id);

		// Shell command used to copy files.
		$command = apply_filters('copy_blog_files_command', sprintf("cp -Rfp %s %s --exclude sites", $from, $to), $from, $to );
		if(!function_exists('exec')) {
		    wp_die('Function exce not supported. Could not copy images / files. Pleae copy without or contact your hoster');
		}
		exec($command);
		// exec('rm ' . $to . 'sites -R');
	}

	/**
	 * Replace URLs in post content
	 *
	 * @param int $from_blog_id ID of the blog being copied from.
	 * @param int $to_blog_id ID of the blog being copied to.
	 */
	private function replace_content_urls( $from_blog_id, $to_blog_id ) {
		global $wpdb;
		$to_blog_prefix = $this->get_blog_prefix( $to_blog_id );
		$from_blog_url = get_blog_option( $from_blog_id, 'siteurl' );
		$to_blog_url = get_blog_option( $to_blog_id, 'siteurl' );
		$query = $wpdb->prepare( "UPDATE {$to_blog_prefix}posts SET post_content = REPLACE(post_content, '%s', '%s')", $from_blog_url, $to_blog_url );
		do_action( 'log', $query, 'wordpress-multilingual-multisite');
		$wpdb->query( $query );
	}

	/**
	 * Replace URLs in post GUIDs
	 *
	 * @param int $from_blog_id ID of the blog being copied from.
	 * @param int $to_blog_id ID of the blog being copied to.
	 */
	private function replace_guid_urls( $from_blog_id, $to_blog_id ) {
		global $wpdb;
		$to_blog_prefix = $this->get_blog_prefix( $to_blog_id );
		$from_blog_url = get_blog_option( $from_blog_id, 'siteurl' );
		$to_blog_url = get_blog_option( $to_blog_id, 'siteurl' );
		$query = $wpdb->prepare( "UPDATE {$to_blog_prefix}posts SET guid = REPLACE(guid, '%s', '%s')", $from_blog_url, $to_blog_url );
		do_action( 'log', $query, 'wordpress-multilingual-multisite');
		$wpdb->query( $query );
	}

	/**
	 * Get the database prefix for a blog
	 *
	 * @param int $blog_id ID of the blog.
	 * @return string prefix
	 */
	private function get_blog_prefix( $blog_id ) {
		global $wpdb;
		if( is_callable( array( &$wpdb, 'get_blog_prefix' ) ) ) {
			$prefix = $wpdb->get_blog_prefix( $blog_id );
		} else {
			$prefix = $wpdb->base_prefix . $blog_id . '_';
		}
		return $prefix;
	}
}