<?php

/**
 * The admin-specific functionality of the plugin.
 *
 * @link       https://welaunch.io/plugins/wordpress-multilingual-multisite/
 * @since      1.0.0
 *
 * @package    WordPress_Multilingual_Multisite
 * @subpackage WordPress_Multilingual_Multisite/admin
 */

/**
 * The admin-specific functionality of the plugin.
 *
 * Defines the plugin name, version, and two examples hooks for how to
 * enqueue the admin-specific stylesheet and JavaScript.
 *
 * @package    WordPress_Multilingual_Multisite
 * @subpackage WordPress_Multilingual_Multisite/admin
 * @author     Daniel Barenkamp <support@welaunch.io>
 */
class WordPress_Multilingual_Multisite_Blog_Options extends WordPress_Multilingual_Multisite {

	protected $plugin_name;
	protected $version;
	protected $current_blog_id;
	protected $master_blog_id;

	/**
	 * Initialize the class and set its properties.
	 *
	 * @since    1.0.0
	 * @param      string    $plugin_name       The name of this plugin.
	 * @param      string    $version    The version of this plugin.
	 */
	public function __construct( $plugin_name, $version, $current_blog_id) 
	{
		$this->plugin_name = $plugin_name;
		$this->version = $version;
		$this->current_blog_id = $current_blog_id;
	}

	public function init() 
	{
		global $wordpress_multilingual_multisite;
		$this->options = $wordpress_multilingual_multisite;

		if (!$this->get_option('enable')) {
			return FALSE;
		}

		$masterSite = $this->get_option('masterSite');
		if(empty($masterSite)) {
			return false;
		}

		$this->master_blog_id = $masterSite;

		if($this->current_blog_id == $masterSite) {
			return false;
		}
		
        $overrideSettings = $this->get_option('masterOverrideSettings');
        if(empty($overrideSettings)) {
            return false;
        }

        $overrideSettings = explode(',', $overrideSettings);

        $this->currentSetting = '';
        foreach ($overrideSettings as $overrideSetting) {

        	$this->currentSetting = $overrideSetting;

        	add_filter('pre_option_' . $overrideSetting, array($this, 'get_master_option'), 10, 3);
        	add_filter('theme_mod_' . $overrideSetting, array($this, 'get_master_theme_mod'), 10, 1);
        }
	}

	/**
	 * [add_blog_to_translation_table description]
	 * @author Daniel Barenkamp
	 * @version 1.0.0
	 * @since   1.0.0
	 * @link    https://www.welaunch.io
	 * @param   [type]                       $blog_id [description]
	 * @param   [type]                       $user_id [description]
	 * @param   [type]                       $domain  [description]
	 * @param   [type]                       $path    [description]
	 * @param   [type]                       $site_id [description]
	 * @param   [type]                       $meta    [description]
	 */
	public function add_options_to_new_blog( $site ) 
	{	
		$blog_id = $site->blog_id;
		$locale = "";
		$language = "";
		$country = "";

		$language = get_blog_option($blog_id, 'WPLANG');

		if(empty($language)) {
			$language = "en_US";
		}

		if(strlen($language) > 5) {
			$language = substr( $language, 0, 5);
		}

		if(strlen($language) == 2) {
			$locale = $language . '_' . strtoupper($language);
			$country = strtoupper($language);
			$language = strtolower($language);
			$alternate = $language;

		} elseif(strlen($language) == 5) {
			$locale = $language;
			$country = substr( strtoupper($language), -2);
			$language = substr( strtolower($language), 0, 2);
			if($locale == "en_UN") {
				$alternate = 'x-default';
			} else {
				$alternate = str_replace('_', '-', $locale);
			}
		}

		add_blog_option($blog_id, 'we_locale', $locale);
		add_blog_option($blog_id, 'we_language', $language);
		add_blog_option($blog_id, 'we_alternate', $alternate);
		add_blog_option($blog_id, 'we_country', $country);
	}

	/**
	 * Add new fields to wp-admin/options-general.php page
	 */
	public function register_fields() {

		// Locale Field
		register_setting( 'general', 'we_locale', 'esc_attr' );
		add_settings_field(
			'we_locale',
			'<label for="we_locale">' . esc_html__( 'Locale (e.g. en_US)' , 'wordpress-multilingual-multisite' ) . '</label>',
			array( $this, 'locale_html' ),
			'general'
		);

		// Language
		register_setting( 'general', 'we_language', 'esc_attr' );
		add_settings_field(
			'we_language',
			'<label for="we_language">' . esc_html__( 'Language' , 'wordpress-multilingual-multisite' ) . '</label>',
			array( $this, 'language_html' ),
			'general'
		);

		// Country
		register_setting( 'general', 'we_country', 'esc_attr' );
		add_settings_field(
			'we_country',
			'<label for="we_country">' . esc_html__( 'Country' , 'wordpress-multilingual-multisite' ) . '</label>',
			array( $this, 'country_html' ),
			'general'
		);

		// Language
		register_setting( 'general', 'we_alternate', 'esc_attr' );
		add_settings_field(
			'we_alternate',
			'<label for="we_alternate">' . esc_html__( 'Alternate (e.g. en-US or x-default for international)' , 'wordpress-multilingual-multisite' ) . '</label>',
			array( $this, 'alternate_html' ),
			'general'
		);

		// Custom Blog Name
		register_setting( 'general', 'we_country_selector_name', 'esc_attr' );
		add_settings_field(
			'we_country_selector_name',
			'<label for="we_country_selector_name">' . esc_html__( 'Custom Country Selector Name (fallback: blogname)' , 'wordpress-multilingual-multisite' ) . '</label>',
			array( $this, 'country_selector_name_html' ),
			'general'
		);

		// Custom Flag
		register_setting( 'general', 'we_country_selector_flag_url', 'esc_attr' );
		add_settings_field(
			'we_country_selector_flag_url',
			'<label for="we_country_selector_flag_url">' . esc_html__( 'Custom Country Selector Flag Image URL' , 'wordpress-multilingual-multisite' ) . '</label>',
			array( $this, 'country_selector_flag_url_html' ),
			'general'
		);
	}

	/**
	 * HTML for Locale
	 */
	public function locale_html() {
		$value = get_option( 'we_locale', '' );
		echo '<input type="text" id="we_locale" name="we_locale" value="' . esc_attr( $value ) . '" />';
	}

	/**
	 * HTML for Language
	 */
	public function language_html() {
		global $wp_trans_languages;
		$value = get_option( 'we_language', '' );

		echo '<select id="we_language" name="we_language">';
		foreach($wp_trans_languages as $language_key => $language_value) {
			$selected = ($value == $language_key) ? 'selected="selected"' : '';
			echo "<option value='$language_key' $selected>$language_value</option>";
		}
		echo '</select>';
	}

	/**
	 * HTML for Language
	 */
	public function country_html() {
		global $wp_trans_countries;
		$value = get_option( 'we_country', '' );

		echo '<select id="we_country" name="we_country">';
		foreach($wp_trans_countries as $country_key => $country_value) {
			$selected = ($value == $country_key) ? 'selected="selected"' : '';
			echo "<option value='$country_key' $selected>$country_value</option>";
		}
		echo '</select>';
	}

	/**
	 * HTML for Alternate
	 */
	public function alternate_html() {
		$value = get_option( 'we_alternate', '' );
		echo '<input type="text" id="we_alternate" name="we_alternate" value="' . esc_attr( $value ) . '" />';
	}

	/**
	 * HTML for Alternate
	 */
	public function country_selector_name_html() {
		$value = get_option( 'we_country_selector_name', '' );
		echo '<input type="text" id="we_country_selector_name" name="we_country_selector_name" value="' . esc_attr( $value ) . '" />';
	}

	/**
	 * HTML for Alternate
	 */
	public function country_selector_flag_url_html() {
		$value = get_option( 'we_country_selector_flag_url', '' );
		echo '<input type="text" id="we_country_selector_flag_url" name="we_country_selector_flag_url" value="' . esc_attr( $value ) . '" />';
	}




	public function get_master_option($option_value, $option, $default)
	{
		global $wpdb;

		switch_to_blog($this->master_blog_id);

		$alloptions = wp_load_alloptions();

		if ( isset( $alloptions[$option] ) ) {
            $value = $alloptions[$option];
        } else {
            $value = wp_cache_get( $option, 'options' );
 
            if ( false === $value ) {
                $row = $wpdb->get_row( $wpdb->prepare( "SELECT option_value FROM $wpdb->options WHERE option_name = %s LIMIT 1", $option ) );
 
                // Has to be get_row instead of get_var because of funkiness with 0, false, null values
                if ( is_object( $row ) ) {
                    $value = $row->option_value;
                    wp_cache_add( $option, $value, 'options' );
                } else { // option does not exist, so we must cache its non-existence
                    if ( ! is_array( $notoptions ) ) {
                         $notoptions = array();
                    }
                    $notoptions[$option] = true;
                    wp_cache_set( 'notoptions', $notoptions, 'options' );
 
                    /** This filter is documented in wp-includes/option.php */
                    return apply_filters( "default_option_{$option}", $default, $option, $passed_default );
                }
            }
        }

        restore_current_blog();

        return maybe_unserialize( $value );
	}

	public function get_master_theme_mod($option)
	{
		if($this->master_blog_id == get_current_blog_id()){
			return $option;
		}

		switch_to_blog($this->master_blog_id);
		$masterThemeMod = get_theme_mod($this->currentSetting);
		if(!empty($masterThemeMod)) {
			$option = $masterThemeMod;
		}

        restore_current_blog();

        return maybe_unserialize( $option );
	}
}