<?php

use Google\Cloud\Translate\TranslateClient as Google;
use BabyMarkt\DeepL\DeepL as DeepL;

/**
 * The admin-specific functionality of the plugin.
 *
 * @link       https://welaunch.io/plugins/wordpress-multilingual-multisite/
 * @since      1.0.0
 *
 * @package    WordPress_Multilingual_Multisite
 * @subpackage WordPress_Multilingual_Multisite/admin
 * https://github.com/Stichoza/google-translate-php
 * https://github.com/GoogleCloudPlatform/google-cloud-php-translate
 * https://github.com/Baby-Markt/deepl-php-lib
 * https://github.com/MicrosoftTranslator/Text-Translation-API-V3-PHP/blob/master/Translate.php
 */

/**
 * The admin-specific functionality of the plugin.
 *
 * Defines the plugin name, version, and two examples hooks for how to
 * enqueue the admin-specific stylesheet and JavaScript.
 *
 * @package    WordPress_Multilingual_Multisite
 * @subpackage WordPress_Multilingual_Multisite/admin
 * @author     Daniel Barenkamp <support@welaunch.io>
 */
class WordPress_Multilingual_Multisite_Machine_Translation extends WordPress_Multilingual_Multisite {

    protected $plugin_name;
    protected $version;
    protected $current_blog_id;
    protected $translation_provider;
    protected $options;

    /**
     * Initialize the class and set its properties.
     *
     * @since    1.0.0
     * @param      string    $plugin_name       The name of this plugin.
     * @param      string    $version    The version of this plugin.
     */
    public function __construct( $plugin_name, $version, $current_blog_id) 
    {
        $this->plugin_name = $plugin_name;
        $this->version = $version;
        $this->current_blog_id = $current_blog_id;
    }

    public function init() {

        global $wordpress_multilingual_multisite;
        $this->options = $wordpress_multilingual_multisite;
        $this->translation_provider = $this->get_option('machineServiceProvider');

        if(!$this->get_option('machineEnable')) {
            return;
        }

        $post_types = get_post_types();
        foreach ($post_types as $post_type) {
            add_filter( 'bulk_actions-edit-' . $post_type, array($this, 'add_bulk_action'), 20, 1 );
            add_filter( 'handle_bulk_actions-edit-' . $post_type, array($this, 'handle_bulk_action'), 10, 3 );
        }

        $taxonomies = get_taxonomies();
        foreach ($taxonomies as $taxonomy) {
            add_action( $taxonomy . '_edit_form', array( $this, 'machine' ), 80, 1 );
            add_action( 'edited_' . $taxonomy, array( $this, 'translate_single_term' ), 10, 2 );

            add_filter( 'bulk_actions-edit-' . $taxonomy, array($this, 'add_bulk_action'), 20, 1 );
            add_filter( 'handle_bulk_actions-edit-' . $taxonomy, array($this, 'handle_bulk_action'), 10, 3 );
        }

        add_filter('bulk_actions-edit-comments', array($this, 'add_bulk_action'), 20, 1 );
        add_filter('handle_bulk_actions-edit-comments',array($this, 'handle_bulk_action'), 10, 3 );
    }

    /**
     * Add custom ticket metaboxes
     * @author Daniel Barenkamp
     * @version 1.0.0
     * @since   1.0.0
     * @link    https://plugins.db-dzine.com
     * @param   [type]                       $post_type [description]
     * @param   [type]                       $post      [description]
     */
    public function add_custom_metaboxes($post_type, $post)
    {
        if(!$this->get_option('machineEnable')) {
            return;
        }

        if($this->get_current_user_role() == "translator") {
            return false;
        }
        add_meta_box('wordpress-multilingual-multisite-machine', esc_html__('Machine Translation', 'wordpress-multilingual-multisite'), array($this, 'machine'), '', 'side', 'high');
    }

    /**
     * Display Metabox machine
     * @author Daniel Barenkamp
     * @version 1.0.0
     * @since   1.0.0
     * @link    https://plugins.db-dzine.com
     * @return  [type]                       [description]
     */
    public function machine()
    {
        global $post, $term, $blogs_data, $wp_trans_languages;

        $default_language = $this->get_option('defaultSourceLanguage');

        $current_language = '';
        if(isset($blogs_data[$this->current_blog_id])) {
            $current_language = $blogs_data[$this->current_blog_id]['language'];
        }
        $source_language_options = "";
        $target_language_options = "";
        foreach ($wp_trans_languages as $language_key => $language_name) {

            $selected = '';
            if($language_key ==  $default_language) {
                $selected = 'selected="selected"';
            }
            $source_language_options .= '<option ' . $selected . ' value="' . $language_key . '">' . $language_name . '</option>';

            $selected = '';
            if($language_key == $current_language) {
                $selected = 'selected="selected"';
            }
            $target_language_options .= '<option ' . $selected . ' value="' . $language_key . '">' . $language_name . '</option>';
        }
        
        ?>
        <div class="wordpress-multilingual-multisite-container">

            <?php 
            if(!empty($post)) { 
                echo '<input type="hidden" name="post_id" value="' . $post->ID . '">';
            } else {
                echo '<input type="hidden" name="term_id" value="' . $term->term_id . '">';
                echo '<hr>';
                echo '<h4>' . esc_html__('Machine Translation', 'wordpress-multilingual-multisite') . '</h4>';
            }
            ?>

            <div class="wordpress-multilingual-multisite-row">
                <label for="machine_source_language"><?php esc_html_e('Source Language', 'wordpress-multilingual-multisite') ?></label>
                <select name="machine_source_language">
                    <option value=""><?php esc_html_e('Select Source Language', 'wordpress-multilingual-multisite') ?></option>
                    <?php echo $source_language_options  ?>
                </select>
            </div>

            <div class="wordpress-multilingual-multisite-row">
                <label for="machine_target_language"><?php esc_html_e('Target Language', 'wordpress-multilingual-multisite') ?></label>
                <select name="machine_target_language">
                    <option value=""><?php esc_html_e('Select Target Language', 'wordpress-multilingual-multisite') ?></option>
                    <?php echo $target_language_options  ?>
                </select>
            </div>

            <div class="wordpress-multilingual-multisite-row">
                <label for="machine_translation">
                    <input type="checkbox" name="machine_translation"><?php echo sprintf( esc_html__('Translate via %s', 'wordpress-multilingual-multisite'), wp_kses_post($this->translation_provider)  ) ?>
                </label>
            </div>
            
        </div>

        <?php
    }

    /**
     * Save Custom Metaboxes
     * @author Daniel Barenkamp
     * @version 1.0.0
     * @since   1.0.0
     * @link    https://plugins.db-dzine.com
     * @param   [type]                       $post_id [description]
     * @param   [type]                       $post    [description]
     * @return  [type]                                [description]
     */
    public function translate_single_post($post_id, $post, $update)
    {
        global $post;
        
        if(!is_a($post, 'WP_Post')) {

            return false;
        }

        if(!$this->get_option('machineEnable')) {
            return;
        }

        if (!current_user_can('edit_post', $post->ID)) {
            return $post->ID;
        }

        if ($post->post_type == 'revision') {
            return false;
        }

        if(!isset($_POST['machine_translation'])) {
            return false;
        }

        if(
            !isset($_POST['machine_source_language']) 
            || !isset($_POST['machine_target_language']) 
            || empty($_POST['machine_source_language']) 
            || empty($_POST['machine_target_language'])) {
            return flase;
        }

        $source_language = $_POST['machine_source_language'];
        $target_language = $_POST['machine_target_language'];

        $this->translate_post( array($post), $source_language, $target_language);

        wp_redirect( admin_url( 'post.php?action=edit&post=' . $post->ID . '&post_type=' . $post->post_type) );
        exit;
    }

    public function translate_multiple_posts()
    {
        global $blogs_data;

        if(!isset($_GET['wp-trans-machine']) || $_GET['wp-trans-machine'] !== "post" ||
            !isset($_GET['post_ids']) || empty($_GET['post_ids'])
        ) {
            return false;
        }

        $post_ids = array_map('absint', explode(',', $_GET['post_ids']));

        $source_language = $this->get_option('defaultSourceLanguage');
        $target_language = $blogs_data[$this->current_blog_id]['language'];

        if(empty($source_language) || empty($target_language)) {
            wp_die('Source or Target Language missing.');
        }

        $posts = array();
        foreach ($post_ids as $post_id) {
            $posts[] = get_post($post_id);
        }

        $this->translate_post( $posts, $source_language, $target_language);
        
        wp_redirect( admin_url( 'edit.php?post_type=' . $_GET['post_type']) );
        exit;
    }

    public function translate_post($posts, $source_language, $target_language)
    {
        if(!$this->translation_provider) {
            wp_die('No Translation Provider set');
        }

        // Prevent infinite loop
        remove_action('save_post', array($this, 'translate_single_post'), 1);

        switch_to_blog($this->current_blog_id);

        $data_to_translate = $this->get_option('machinePostData');
        $meta_to_translate = explode(',', $this->get_option('machinePostMeta'));

        if(empty($data_to_translate) && empty($meta_to_translate)) {
            wp_die('No post data or meta keys set in machine translation settings.');
        }

        $original_data = array();
        $translated = array();
        $translated_metas = array();
        $segmentData = $this->get_option('machineSegmentData');

        $this->set_translation_client();

        foreach ($posts as $post) {      

            $translated['ID'] = $post->ID;

            try {
                if($data_to_translate) {

                    foreach ($data_to_translate as $data_key) {
                        $translated[$data_key] = "";

                        $data = $post->$data_key;

                        if(empty($data)) {
                            continue;
                        }

                        if($data_key == "post_content" && $segmentData) {

                            if(!$this->get_option('machineAvoidAutoParagraph')) {
                                $data = wpautop($data);
                            }

                            $translated[$data_key] = $this->translate_and_segment_data($source_language, $target_language, $data);
                        } else {
                            $translated[$data_key] = $this->translate_data($source_language, $target_language, $data);
                        }

                        $translated[$data_key] = $this->fixTranslationIssue($translated[$data_key]);
                    }
                }

                if($meta_to_translate) {
                    foreach ($meta_to_translate as $meta_key_to_translate) {
                        $translated_metas[$meta_key_to_translate] = "";

                        $data = get_post_meta($post->ID, $meta_key_to_translate, true);
                        if(empty($data)) {
                            continue;
                        }

                        if($segmentData && $meta_key_to_translate == "_elementor_data") {

                            $elementorData = json_decode($data);

                            if(!empty($elementorData)) {
                                foreach($elementorData as $elementorDataKey => $elementorDataValue) {
                                    $elementorData[$elementorDataKey] = $this->translate_elementor_element($source_language, $target_language, $elementorDataValue);
                                }   

                                // serialize
                                $translated_metas[$meta_key_to_translate] = json_encode( $elementorData, JSON_UNESCAPED_UNICODE );
                            }


                        } elseif(is_array($data)) {
                            $tmp = array();
                            foreach($data as $dataOrigKey => $dataOrigValue) {
                                $tmp[$dataOrigKey] = $this->translate_data($source_language, $target_language, $dataOrigValue);
                                $tmp[$dataOrigKey] = $this->fixTranslationIssue($tmp[$dataOrigKey]);
                            }
                            $translated_metas[$meta_key_to_translate] = $tmp;
                        } else {
                            $translated_metas[$meta_key_to_translate] = $this->translate_data($source_language, $target_language, $data);
                            $translated_metas[$meta_key_to_translate] = $this->fixTranslationIssue($translated_metas[$meta_key_to_translate]);
                        }
                    }
                }

            } catch (Exception $e) {
                if($this->get_option('machineDebugMode')) {
                    wp_die('Message: ' . $e->getMessage());
                }
            }

            $translated = array_filter($translated);
            if(!empty($translated)) {
                wp_update_post($translated, $post->ID);
            }

            if(!empty($translated_metas)){
                foreach ($translated_metas as $translated_meta_key => $translated_meta) {
                    update_post_meta($post->ID, $translated_meta_key, $translated_meta);
                }
            }
        }
    }

    protected function translate_elementor_element($source_language, $target_language, &$element) 
    {

        if(isset($element->settings)) {

            if(isset($element->settings->title)) {
                $element->settings->title = $this->translate_data($source_language, $target_language, $element->settings->title);
            }

            if(isset($element->settings->text)) {
                $element->settings->text = $this->translate_data($source_language, $target_language, $element->settings->text);
            }

            if(isset($element->settings->editor)) {
                $element->settings->editor = $this->translate_and_segment_data($source_language, $target_language, $element->settings->editor);
            }
        }

        if(isset($element->elements)) {
            foreach($element->elements as $elementKey => $elementValue) {
                $element->elements[$elementKey] = $this->translate_elementor_element($source_language, $target_language, $elementValue);
            }
        }

        return $element;
    }

    protected function translate_and_segment_data($source_language, $target_language, $data)
    {
        $data = preg_replace("/<a href=.*?>(.*?)<\/a>/", "$1", $data);
        $data = html_entity_decode($data);

        $random = strtoupper(dechex(rand(0,10000000000)));
        // replace comment starts
        $data = preg_replace('/<!--/', '@MARKER-START-'. $random.'@', $data);
        // replace comment ends
        $data = preg_replace('/-->/', '@MARKER-END-'. $random.'@', $data);
        // strip all data tags

        $data = str_replace( array('<br>', '<br />'), ' ' , $data);

        $preservedKeys = $this->get_option('machinePreservedKeys');
        if(empty($preservedKeys)) {
            $preservedKeys = '<img><ol><ul><li><p><h1><h2><h3><h4><h5><h6><span><table><tr><td><thead><tbody><tfoot><th><figure><figcaption><blockquote><em><cite><iframe>';
        }

        $data = strip_tags($data, $preservedKeys);
        // replace back comment starts
        $data = preg_replace('/@MARKER-START-'. $random.'@/', '<!--', $data);
        // replace back comment ends
        $data = preg_replace('/@MARKER-END-'. $random.'@/', '-->', $data);

        $orig = $data;

        $_data = array();

        // get text="" in shortcodes 
        $textShortcodes = preg_match_all('/text="(.+?)"/', $data, $matches);
        if(isset($matches[0]) && !empty($matches[0]) & isset($matches[1]) && !empty($matches[1])) {
            $_data = array_merge($_data, $matches[1]);
        }

        $textShortcodes = preg_match_all('/label="(.+?)"/', $data, $matches);
        if(isset($matches[0]) && !empty($matches[0]) & isset($matches[1]) && !empty($matches[1])) {
            $_data = array_merge($_data, $matches[1]);
        }

        $tooltipShortcodes = preg_match_all('/tooltip="(.+?)"/', $data, $matches);
        if(isset($matches[0]) && !empty($matches[0]) & isset($matches[1]) && !empty($matches[1])) {
            $_data = array_merge($_data, $matches[1]);
        }

        $tooltipShortcodes = preg_match_all('/heading="(.+?)"/', $data, $matches);
        if(isset($matches[0]) && !empty($matches[0]) & isset($matches[1]) && !empty($matches[1])) {
            $_data = array_merge($_data, $matches[1]);
        }

        $tooltipShortcodes = preg_match_all('/title="(.+?)"/', $data, $matches);
        if(isset($matches[0]) && !empty($matches[0]) & isset($matches[1]) && !empty($matches[1])) {
            $_data = array_merge($_data, $matches[1]);
        }

        $tooltipShortcodes = preg_match_all('/t_prefix="(.+?)"/', $data, $matches);
        if(isset($matches[0]) && !empty($matches[0]) & isset($matches[1]) && !empty($matches[1])) {
            $_data = array_merge($_data, $matches[1]);
        }
       
        
        // remove shortcodes
        $data = strip_tags( preg_replace("/\[[^\]]+\]/", '', $data), '<p>' );

        // divi stuff
        if($this->get_option('machineDiviSupport')) {
            $pTags = preg_match_all('#<\s*?p\b[^>]*>(.*?)</p\b[^>]*>#s', $data, $matches);
            if(isset($matches[0]) && !empty($matches[0]) & isset($matches[1]) && !empty($matches[1])) {
                $_data = array_merge($_data, $matches[1]);

                foreach($matches[0] as $pTagDataToRemove) {
                    $data = str_replace($pTagDataToRemove, '', $data);
                }
            }
        }

        // now remove all ...
        $data = strip_tags($data);

        // $data = wpautop($data);
        // $dom = new DOMDocument();
        // $dom->loadHTML($data);

        
        // $_dataToRemove = array();
        // foreach($dom->getElementsByTagName('*') as $element ){

        //     // Skip HTMl & Body Nodes
        //     if($element->tagName == "html" || $element->tagName == "body") {
        //         continue;
        //     }

        //     $domKey = md5($element->nodeValue);

        //     // Strip Shortcodes
        //     $checkParentNode = $element->parentNode;

        //     if($checkParentNode->tagName != "html" && $checkParentNode->tagName != "body") {

        //         $parentKey = md5($checkParentNode->nodeValue);
        //         if(!isset($_dataToRemove[ $parentKey ])) {
        //             $_dataToRemove[ md5($checkParentNode->nodeValue) ] = array(
        //                  $element->nodeValue
        //             );
        //         } else {
        //             $_dataToRemove[ md5($checkParentNode->nodeValue) ][] = $element->nodeValue;
        //         }
               
        //     }

        //     $_data[$domKey] = $element->nodeValue;
        // }

        // if(!empty($_dataToRemove)) {
        //     foreach ($_dataToRemove as $domKeyToCheck => $dataToRemove) {
        //         if(isset( $_data[$domKeyToCheck] ) && !empty( $_data[$domKeyToCheck] )) {
        //             $_data[$domKeyToCheck] = str_replace($dataToRemove, '', $_data[$domKeyToCheck]);
        //         }
        //     }
        // }

        // if(empty($_data)) {
        //     continue;
        // }

        $data = $this->multiexplode( array(PHP_EOL, '.', '?', '!', '"', '“'), $data);

        // many wp posts are saved without starting tags.
        // $data = preg_match_all('/>(.*?)<\//s', $data, $match);

        $_data = array_merge($_data, $data);

        $_data = array_unique( array_filter( array_map('trim', array_unique( $_data ) ) ) );

        // Sort by length
        usort($_data, function($a, $b) {
            return strlen($b) - strlen($a);
        });

        $translatedContent = array();

        foreach ($_data as $_dataTrans) {

            $_dataTrans = strip_tags($_dataTrans);

            $_dataTrans = preg_replace('/[\x00-\x1F\x7F\xA0]/u', '', $_dataTrans);

            // $_dataTrans = utf8_decode($_dataTrans);
            $_dataTranslated = $this->translate_data($source_language, $target_language, $_dataTrans);
            $_dataTrans = trim( $_dataTrans );
            $_dataTranslated = trim(  $_dataTranslated );
            // Has removed . from paragraphs
            $_dataTranslated = str_replace( array('.', '?', '!'), '' , $_dataTranslated);                          

            $orig = str_replace( $_dataTrans, $_dataTranslated, $orig);

        }

        $orig = preg_replace('/[\x00-\x1F\x7F\xA0]/u', '', $orig); 

        return $orig;
    }

    function multiexplode ($delimiters,$string) {

        $ready = str_replace($delimiters, $delimiters[0], $string);
        $launch = explode($delimiters[0], $ready);
        return  $launch;
    }

    /**
     * Save Custom Metaboxes
     * @author Daniel Barenkamp
     * @version 1.0.0
     * @since   1.0.0
     * @link    https://plugins.db-dzine.com
     * @param   [type]                       $post_id [description]
     * @param   [type]                       $post    [description]
     * @return  [type]                                [description]
     */
    public function translate_single_term($term_id, $tt_id)
    {
        global $term;


        if(!$this->get_option('machineEnable')) {
            return;
        }

        if(!isset($_POST['machine_translation'])) {
            return false;
        }

        if(
            !isset($_POST['machine_source_language']) 
            || !isset($_POST['machine_target_language']) 
            || empty($_POST['machine_source_language']) 
            || empty($_POST['machine_target_language'])) {
            return flase;
        }

        $source_language = $_POST['machine_source_language'];
        $target_language = $_POST['machine_target_language'];

        $this->translate_terms( array( get_term($term_id)), $source_language, $target_language);

        wp_redirect( admin_url( 'post.php?action=edit&post=' . $post->ID . '&post_type=' . $post->post_type) );
        exit;
    }

    /**
     * Save Custom Metaboxes
     * @author Daniel Barenkamp
     * @version 1.0.0
     * @since   1.0.0
     * @link    https://plugins.db-dzine.com
     * @param   [type]                       $post_id [description]
     * @param   [type]                       $post    [description]
     * @return  [type]                                [description]
     */
    public function translate_multiple_terms()
    {

        global $blogs_data;

        if(!isset($_GET['wp-trans-machine']) || $_GET['wp-trans-machine'] !== "term" ||
            !isset($_GET['term_ids']) || empty($_GET['term_ids'])
        ) {
            return false;
        }
        
        $term_ids = array_map('absint', explode(',', $_GET['term_ids']));

        $source_language = $this->get_option('defaultSourceLanguage');
        $target_language = $blogs_data[$this->current_blog_id]['language'];

        if(empty($source_language) || empty($target_language)) {
            wp_die('Source or Target Language missing.');
        }

        $terms = array();
        foreach ($term_ids as $term_id) {
            $terms[] = get_term($term_id);
        }

        $this->translate_terms( $terms, $source_language, $target_language);

        wp_redirect( admin_url( 'edit-tags.php?taxonomy=' . $_GET['taxonomy'] . '&post_type=' . $_GET['post_type']) );
        exit;
    }

    public function translate_terms($terms, $source_language, $target_language)
    {
        if(!$this->translation_provider) {
            wp_die('No Translation Provider set');
        }

        $data_to_translate = $this->get_option('machineTermData');
        if(!empty($data_to_translate)) {
            $data_to_translate = $data_to_translate;
        }

        $meta_to_translate = $this->get_option('machineTermMeta');
        if(!empty($meta_to_translate)) {
            $meta_to_translate = explode(',', $meta_to_translate);
        }

        $original_data = array();
        $translated = array();
        $translated_metas = array();

        $segmentData = $this->get_option('machineSegmentData');

        $this->set_translation_client();

        foreach ($terms as $term) {      

            // Prevent infinite loop
            remove_action('edited_' . $term->taxonomy, array($this, 'translate_single_term'), 10);

            $translated['ID'] = $term->term_id;

            try {
                if($data_to_translate) {
                    foreach ($data_to_translate as $data_key) {
                        
                        $translated[$data_key] = "";

                        $data = $term->$data_key;
                        if(empty($data)) {
                            continue;
                        }

                        if($data_key == "description" && $segmentData) {

                            $orig = $data;

                            $data = preg_replace("/\[[^\]]+\]/", '', $data);
                            $data = wpautop($data);
                            $dom = new DOMDocument();
                            $dom->loadHTML($data);
                            
                            $_data = array();
                            foreach($dom->getElementsByTagName('*') as $element ){

                                // Skip HTMl & Body Nodes
                                if($element->tagName == "html" || $element->tagName == "body") {
                                    continue;
                                }

                                // Strip Shortcodes
                                $_data[] = preg_replace("/\[[^\]]+\]/", '', $element->nodeValue);
                            }

                            if(empty($_data)) {
                                continue;
                            }

                            $_data = array_unique($_data);

                            $translatedContent = array();
                            foreach ($_data as $_dataTrans) {
                                $_dataTrans = utf8_decode($_dataTrans);
                                $_dataTranslated = $this->translate_data($source_language, $target_language, $_dataTrans);
                                $orig = str_replace(htmlspecialchars( $_dataTrans ), htmlspecialchars( $_dataTranslated ), $orig);
                            }
                        
                            $translated[$data_key] = $orig;
                        } else {
                            $translated[$data_key] = $this->translate_data($source_language, $target_language, $data);
                        }

                        $translated[$data_key] = $this->fixTranslationIssue($translated[$data_key]);
                    }
                }

                if($meta_to_translate) {
                    foreach ($meta_to_translate as $meta_key_to_translate) {

                        $translated_metas[$meta_key_to_translate] = "";

                        $data = get_term_meta($term->term_id, $meta_key_to_translate, true);
                        if(empty($data)) {
                            continue;
                        }
                        $translated_metas[$meta_key_to_translate] = $this->translate_data($source_language, $target_language, $data);
                        $translated_metas[$meta_key_to_translate] = $this->fixTranslationIssue($translated_metas[$meta_key_to_translate]);
                    }
                }
            } catch (Exception $e) {
                if($this->get_option('machineDebugMode')) {
                    wp_die('Message: ' . $e->getMessage());
                }
            }

            $translated = array_filter($translated);
            if(!empty($translated)) {
                wp_update_term($term->term_id, $term->taxonomy, $translated);
            }

            if(!empty($translated_metas)){
                foreach ($translated_metas as $translated_meta_key => $translated_meta) {
                    update_term_meta($term->term_id, $translated_meta_key, $translated_meta);
                }
            }
        }
    }

    /**
     * Save Custom Metaboxes
     * @author Daniel Barenkamp
     * @version 1.0.0
     * @since   1.0.0
     * @link    https://plugins.db-dzine.com
     * @param   [type]                       $post_id [description]
     * @param   [type]                       $post    [description]
     * @return  [type]                                [description]
     */
    public function translate_multiple_comments()
    {

        global $blogs_data;

        if(!isset($_GET['wp-trans-machine']) || $_GET['wp-trans-machine'] !== "comments" ||
            !isset($_GET['comments']) || empty($_GET['comments'])
        ) {
            return false;
        }
        
        $comment_ids = array_map('absint', explode(',', $_GET['comments']));

        $source_language = $this->get_option('defaultSourceLanguage');
        $target_language = $blogs_data[$this->current_blog_id]['language'];

        if(empty($source_language) || empty($target_language)) {
            wp_die('Source or Target Language missing.');
        }

        $comments = array();
        foreach ($comment_ids as $comment_id) {
            $comment = get_comment($comment_id);
            if(!$comment) {
                continue;
            }
            $comments[] = $comment;
        }

        $this->translate_comments( $comments, $source_language, $target_language);

        wp_redirect( admin_url( 'edit-comments.php') );
        exit;
    }

    public function translate_comments($comments, $source_language, $target_language)
    {
        if(!$this->translation_provider) {
            wp_die('No Translation Provider set');
        }

        $data_to_translate = array(
            'comment_content',
        );

        $original_data = array();
        $translated = array();
        $translated_metas = array();

        $segmentData = $this->get_option('machineSegmentData');

        $this->set_translation_client();

        foreach ($comments as $comment) {      

            $translated['comment_ID'] = $comment->comment_ID;

            try {
                if($data_to_translate) {
                    foreach ($data_to_translate as $data_key) {
                        
                        $translated[$data_key] = "";

                        $data = $comment->$data_key;
                        if(empty($data)) {
                            continue;
                        }

                        if($data_key == "comment_content" && $segmentData) {

                            $orig = $data;

                            $data = preg_replace("/\[[^\]]+\]/", '', $data);
                            $data = wpautop($data);
                            $dom = new DOMDocument();
                            $dom->loadHTML($data);
                            
                            $_data = array();
                            foreach($dom->getElementsByTagName('*') as $element ){

                                // Skip HTMl & Body Nodes
                                if($element->tagName == "html" || $element->tagName == "body") {
                                    continue;
                                }

                                // Strip Shortcodes
                                $_data[] = preg_replace("/\[[^\]]+\]/", '', $element->nodeValue);
                            }

                            if(empty($_data)) {
                                continue;
                            }

                            $_data = array_unique($_data);

                            $translatedContent = array();
                            foreach ($_data as $_dataTrans) {
                                $_dataTrans = utf8_decode($_dataTrans);
                                
                                $_dataTranslated = $this->translate_data($source_language, $target_language, $_dataTrans);
                                $orig = str_replace(htmlspecialchars( $_dataTrans ), htmlspecialchars( $_dataTranslated ), $orig);
                            }
                        
                            $translated[$data_key] = $orig;
                        } else {
                            $translated[$data_key] = $this->translate_data($source_language, $target_language, $data);
                        }

                        $translated[$data_key] = $this->fixTranslationIssue($translated[$data_key]);
                    }
                }

            } catch (Exception $e) {
                if($this->get_option('machineDebugMode')) {
                    wp_die('Message: ' . $e->getMessage());
                }
            }

            $translated = array_filter($translated);
            if(!empty($translated)) {
                wp_update_comment($translated);
            }
        }
    }

    protected function set_translation_client()
    {
        // Google Translate (Paid)
        if($this->translation_provider == "Google") {

            if(!$this->get_option('machineGoogleAPIKey')) {
                wp_die('No Google API Key Set.');
            }
            $this->translationClient = new Google([
                'key' => trim( $this->get_option('machineGoogleAPIKey') )
            ]);

        // DeepL Client
        } elseif($this->translation_provider == "DeepL") {

            if(!$this->get_option('machineDeepLAPIKey')) {
                wp_die('No DeepL API Key Set.');
            }

            $host = 'api.deepl.com';
            if($this->get_option('machineDeepLFree')) {
                $host = 'api-free.deepl.com';
            }

            $this->translationClient = new DeepL( trim( $this->get_option('machineDeepLAPIKey') ), 2, $host);
        }
    }

    protected function translate_data($source_language, $target_language, $data)
    {
        if(empty($data)) {
            return false;
        }

        // Google Paid
        if($this->translation_provider == "Google") {

            $translated_data = $this->translationClient->translate($data, array(
                'target' => $target_language,
                'source' => $source_language,
            ));

            if(!isset($translated_data['text']) || empty($translated_data['text'])) {
                $translated_data = false;
            } 

            $translated_data = $translated_data['text'];

        // DeepL
        } elseif($this->translation_provider == "DeepL") {
            $translated_data = $this->translationClient->translate($data, $source_language, $target_language);
            if(!isset($translated_data[0]['text']) || empty($translated_data[0]['text'])) {
                $translated_data = false;
            } 

            $translated_data = $translated_data[0]['text'];

        // Microsoft
        } elseif($this->translation_provider == "Microsoft") {
            $translated_data = $this->translate_microsoft($data, $source_language, $target_language);
        }

        if(empty($translated_data)) {
            return false;
        } else {
            return $translated_data;
        }
    }

    public function add_bulk_action($bulk_actions)
    {
        global $blogs_data;

        $bulk_actions['machine_translate'] = sprintf( esc_html__( 'Translate via %s', 'woocommerce-customizations'), $this->translation_provider);

        return $bulk_actions;
    }

    public function handle_bulk_action($redirect_to, $action, $post_ids)
    {
        if ( $action !== "machine_translate" ) {
            return $redirect_to;
        }

        $blog_url = admin_url('edit.php?post_type=' . $_GET['post_type']);
        
        if(isset($_GET['delete_comments']) && !empty($_GET['delete_comments'])) {
            $blog_url .= '&wp-trans-machine=comments&comments=' . implode(',', $_GET['delete_comments']);
        } elseif(isset($_GET['taxonomy']) && !empty($_GET['taxonomy'])) {
            $blog_url .= '&wp-trans-machine=term&taxonomy=' . $_GET['taxonomy'] . '&term_ids=' . implode(',', $post_ids);
        } else {
            $blog_url .= '&wp-trans-machine=post&post_ids=' . implode(',', $post_ids);       
        }

        return $blog_url;
    }

    /**
     * [translate_microsoft description]
     * Adapted from https://github.com/MicrosoftTranslator/Text-Translation-API-V3-PHP
     * @author Daniel Barenkamp
     * @version 1.0.0
     * @since   1.0.0
     * @link    https://plugins.db-dzine.com
     * @param   [type]                       $data            [description]
     * @param   [type]                       $source_language [description]
     * @param   [type]                       $target_language [description]
     * @return  [type]                                        [description]
     */
    protected function translate_microsoft($data, $source_language, $target_language)
    {
        $key = trim( $this->get_option('machineMicrosoftAPIKey') );
        if(!$key) {
            wp_die('No Microsoft API Key set');
        }

        $host = "https://api.cognitive.microsofttranslator.com";
        $path = "/translate?api-version=3.0";

        // Translate to Params.
        $params = "&from=" . $source_language . "&to=" . $target_language;

        $requestBody = array (
            array (
                'Text' => $data,
            ),
        );
        $content = json_encode($requestBody);
        $headers = "Content-type: application/json\r\n" .
            "Content-length: " . strlen($content) . "\r\n" .
            "Ocp-Apim-Subscription-Key: $key\r\n" .
            "X-ClientTraceId: " . $this->com_create_guid() . "\r\n";

        // NOTE: Use the key 'http' even if you are making an HTTPS request. See:
        // http://php.net/manual/en/function.stream-context-create.php
        $options = array (
            'http' => array (
                'header' => $headers,
                'method' => 'POST',
                'content' => $content
            )
        );
        $context  = stream_context_create ($options);
        $result = file_get_contents ($host . $path . $params, false, $context);
        // Note: We convert result, which is JSON, to and from an object so we can pretty-print it.
        // We want to avoid escaping any Unicode characters that result contains. See:
        // http://php.net/manual/en/function.json-encode.php
        // $json = json_encode(json_decode($result), JSON_UNESCAPED_UNICODE | JSON_PRETTY_PRINT);

        if(!$result) {
            die('file_get_contents is not working. Please enable allow_url_fopen in your php.ini');
        }
        $array = json_decode($result, true);

        return $array[0]['translations'][0]['text'];
    }

    private function com_create_guid() {
        return sprintf( '%04x%04x-%04x-%04x-%04x-%04x%04x%04x',
            mt_rand( 0, 0xffff ), mt_rand( 0, 0xffff ),
            mt_rand( 0, 0xffff ),
            mt_rand( 0, 0x0fff ) | 0x4000,
            mt_rand( 0, 0x3fff ) | 0x8000,
            mt_rand( 0, 0xffff ), mt_rand( 0, 0xffff ), mt_rand( 0, 0xffff )
        );
    }

    private function fixTranslationIssue($translated_data)
    {
        $search = array('[/ ', ' = ');
        $replace = array('[/', '=');
        $translated_data = str_replace($search, $replace, $translated_data);
        preg_replace_callback("/(\[[^\]]+\])/", function ($word) {
            return strtolower($word[1]);
        }, $translated_data);
        
        return $translated_data;
    }
}