<?php

/**
 * The admin-specific functionality of the plugin.
 *
 * @link       https://welaunch.io/plugins/wordpress-multilingual-multisite/
 * @since      1.0.0
 *
 * @package    WordPress_Multilingual_Multisite
 * @subpackage WordPress_Multilingual_Multisite/admin
 */

/**
 * The admin-specific functionality of the plugin.
 *
 * Defines the plugin name, version, and two examples hooks for how to
 * enqueue the admin-specific stylesheet and JavaScript.
 *
 * @package    WordPress_Multilingual_Multisite
 * @subpackage WordPress_Multilingual_Multisite/admin
 * @author     Daniel Barenkamp <support@welaunch.io>
 */
class WordPress_Multilingual_Multisite_Sync extends WordPress_Multilingual_Multisite {

	protected $plugin_name;
	protected $version;

	/**
	 * Initialize the class and set its properties.
	 *
	 * @since    1.0.0
	 * @param      string    $plugin_name       The name of this plugin.
	 * @param      string    $version    The version of this plugin.
	 */
	public function __construct( $plugin_name, $version, $translation_tables, $current_blog_id, $post_copier, $terms_copier ) 
    {
        global $sync_running;

        $sync_running = false;

		$this->plugin_name = $plugin_name;
		$this->version = $version;

        $this->translation_tables = $translation_tables;
        $this->post_copier = $post_copier;
        $this->terms_copier = $terms_copier;

        $this->current_blog_id = $current_blog_id;
        $this->prefix = 'wordpress_multilingual_multisite_';
	}

	public function init() 
    {
		global $wordpress_multilingual_multisite;

		$this->options = $wordpress_multilingual_multisite;

		if (!$this->get_option('enable')) {
			return FALSE;
		}
	}

    public function sync_post_meta( $meta_id, $object_id, $meta_key, $_meta_value )
    {

        global $sync_running;
        if(!$this->get_option('syncEnable')) {
            return false;
        }

        // if($sync_running) {
        //     return false;
        // }

        if(isset($_POST['wordpress-multilingual-multisite-import'])) {
            return false;
        }

        $post_type = get_post_type($object_id);
        if(empty($post_type) || !$this->get_option('syncEnable' . $post_type)) {
            return false;
        }

        $masterSite = $this->get_option('masterSite');
        $syncOnlyMaster = $this->get_option('syncOnlyMaster');
        if(!$masterSite && $syncOnlyMaster) {
            wp_die( esc_html__('Sync only for master site enabled, but no master site is set in plugin settings.', 'wordpress-multilingual-multisite') );
        } elseif($syncOnlyMaster && $masterSite && $masterSite != $this->current_blog_id){
            return false;
        }

        if($this->get_option('syncOnlySpecificPosts')) {
            $syncThisPost = get_post_meta($object_id, 'wordpress_multilingual_multisite_sync_post', true);
            if(!$syncThisPost) {
                return false;
            }
        }

        if(!$this->get_option('syncPostMetaAll' . $post_type)) {
            $metas = $this->get_option('syncPostMeta' . $post_type);
            if(empty($metas)) {
                return false;
            }

            $metas = explode(',', $metas);
            if(!in_array($meta_key, $metas)) {
                return;
            }
        }


        $this->translation_tables->set_table('posts_translations');
        $translated_posts = $this->translation_tables->get_translated_items_by_item_id($this->current_blog_id, $object_id);

        if(empty($translated_posts)) {
            return false;
        }

        $excludedSites = $this->get_option('excludedSites');
        $sync_running = 'post_meta';

        global $wpdb;
        foreach ($translated_posts as $blog_id => $post_id) {

            if(empty($post_id) || $blog_id == $this->current_blog_id || !is_numeric($blog_id)) {
                continue;
            }

            if($excludedSites && in_array($blog_id, $excludedSites)) {
                continue;
            }

            switch_to_blog($blog_id);
            update_post_meta($post_id, $meta_key, $_meta_value);

            // Regenerate WC lookup table
            if(in_array($post_type, array('product', 'product_variation'))) {

                $product = wc_get_product($object_id);
                if($product) {
                    $product->save();
                }
            }

            restore_current_blog();
        }

        $sync_running = false;
    }

    public function delete_post_meta( $deleted_meta_ids, $post_id, $meta_key, $_meta_value )
    {
        if(isset($_POST['wordpress-multilingual-multisite-import'])) {
            return FALSE;
        }

        $post_type = get_post_type($post_id);
        if(empty($post_type) || !$this->get_option('syncEnable' . $post_type)) {
            return false;
        }

        $masterSite = $this->get_option('masterSite');
        $syncOnlyMaster = $this->get_option('syncOnlyMaster');
        if(!$masterSite && $syncOnlyMaster) {
            wp_die( esc_html__('Sync only for master site enabled, but not master site is set', 'wordpress-multilingual-multisite') );
        } elseif($syncOnlyMaster && $masterSite && $masterSite != $this->current_blog_id){
            return false;
        }

        if(!$this->get_option('syncPostMetaAll')) {
            $metas = $this->get_option('syncPostMeta' . $post_type);
            if(empty($metas)) {
                return false;
            }

            $metas = explode(',', $metas);
            if(!in_array($meta_key, $metas)) {
                return;
            }
        }

        $this->translation_tables->set_table('posts_translations');
        $translated_posts = $this->translation_tables->get_translated_items_by_item_id($this->current_blog_id, $post_id);

        if(empty($translated_posts)) {
            return false;
        }

        if($this->get_option('syncOnlySpecificPosts')) {
            $syncThisPost = get_post_meta($post_id, 'wordpress_multilingual_multisite_sync_post', true);
            if(!$syncThisPost) {
                return false;
            }
        }

        $excludedSites = $this->get_option('excludedSites');

        foreach ($translated_posts as $blog_id => $post_id) {

            if(empty($post_id) || $blog_id == $this->current_blog_id || !is_numeric($blog_id)) {
                continue;
            }

            if($excludedSites && in_array($blog_id, $excludedSites)) {
                continue;
            }

            switch_to_blog($blog_id);
            delete_post_meta($post_id, $meta_key, $_meta_value);

            // Regenerate WC lookup table
            if(in_array($post_type, array('product', 'product_variation'))) {

                $product = wc_get_product($object_id);
                if($product) {
                    $product->save();
                }
            }

            restore_current_blog();
        }
    }

    public function sync_post_data($originalPostId, $post_after, $post_before)
    {
        global $sync_running;

        if(!$this->get_option('syncEnable')) {
            return false;
        }

        if($sync_running) {
            return false;
        }

        if(isset($_POST['wordpress-multilingual-multisite-import'])) {
            return FALSE;
        }

        $post_type = $post_after->post_type;
        $willPostBeCreate = array('auto-draft', 'draft');
        if($post_after->post_status == "publish" && in_array($post_before->post_status, $willPostBeCreate) && $this->get_option('syncCreate' . $post_type)) {
            return false;
        }

        $masterSite = $this->get_option('masterSite');
        $syncOnlyMaster = $this->get_option('syncOnlyMaster');
        if(!$masterSite && $syncOnlyMaster) {
            wp_die( esc_html__('Sync only for master site enabled, but not master site is set', 'wordpress-multilingual-multisite') );
        } elseif($syncOnlyMaster && $masterSite && $masterSite != $this->current_blog_id){
            return false;
        }

        if(!is_a($post_after, 'WP_Post')) {
            return false;
        }

        if (!current_user_can('edit_post', $post_after->ID)) {
            return $post_after->ID;
        }

        if ($post_type == 'revision') {
            return false;
        }

        if(empty($post_type) || !$this->get_option('syncEnable' . $post_type)) {
            return false;
        }

        if($this->get_option('syncOnlySpecificPosts')) {
            $syncThisPost = get_post_meta($originalPostId, 'wordpress_multilingual_multisite_sync_post', true);
            if(!$syncThisPost) {
                return false;
            }
        }

        if(!$this->get_option('syncPostDataAll' . $post_type)) {
            $data = $this->get_option('syncPostData' . $post_type);
            if(!$data || empty($data)) {
                return false;
            }

            $data = array_flip( explode(',', $data) );
            $updated_post = (array) $post_after;

            $post = array_intersect_key($updated_post, $data);
        } else {
            $post = (array) $post_after;
        }

        unset($post['ID']);

        $this->translation_tables->set_table('posts_translations');
        $translated_posts = $this->translation_tables->get_translated_items_by_item_id($this->current_blog_id, $originalPostId);
        if(empty($translated_posts)) {
            return false;
        }

        unset($translated_posts['id']);
        unset($translated_posts['type']);
        unset($translated_posts['data']);

        $sync_running = 'sync_post_data';
        $excludedSites = $this->get_option('excludedSites');

        foreach ($translated_posts as $blog_id => $post_id) {
            
            if(empty($post_id) || $blog_id == $this->current_blog_id || !is_numeric($blog_id)) {
                continue;
            }

            if($excludedSites && in_array($blog_id, $excludedSites)) {
                continue;
            }

            $post['ID'] = $post_id;

            switch_to_blog($blog_id);
            wp_update_post($post);

            // Regenerate WC lookup table
            if(in_array($post_type, array('product', 'product_variation'))) {

                $product = wc_get_product($post_id);
                if($product) {
                    $product->save();
                }
            }

            restore_current_blog();

            if($this->get_option('syncImages' . $post_type)) {
                $this->post_copier->duplicate_media( $originalPostId, $this->current_blog_id, $post_id, $blog_id);
            }
        }

        $sync_running = false;

        restore_current_blog();
    }

    public function create_post($new_status, $old_status, $post ) 
    { 
        global $sync_running;

        $willPostBeCreate = array('auto-draft', 'draft');
        if(!in_array($old_status, $willPostBeCreate) || $new_status != 'publish') {
            return;
        }

        if(!$this->get_option('syncEnable')) {
            return false;
        }

        if($sync_running) {
            return false;
        }

        if(isset($_POST['wordpress-multilingual-multisite-import'])) {
            return FALSE;
        }

        $masterSite = $this->get_option('masterSite');
        $syncOnlyMaster = $this->get_option('syncOnlyMaster');
        if(!$masterSite && $syncOnlyMaster) {
            wp_die( esc_html__('Sync only for master site enabled, but not master site is set', 'wordpress-multilingual-multisite') );
        } elseif($syncOnlyMaster && $masterSite && $masterSite != $this->current_blog_id){
            return false;
        }

        $post_type = $post->post_type;
        if(empty($post_type) || !$this->get_option('syncEnable' . $post_type)) {
            return false;
        }

        if(!$this->get_option('syncCreate' . $post_type)) {
            return false;
        }

        if($this->get_option('syncOnlySpecificPosts')) {
            $syncThisPost = get_post_meta($post->ID, 'wordpress_multilingual_multisite_sync_post', true);
            if(!$syncThisPost) {
                return false;
            }
        }

        global $blogs_data;

        $excludedSites = $this->get_option('excludedSites');
        $sync_running = 'create_post';

        foreach ($blogs_data as $blog_id => $blog_data) {

            if($this->current_blog_id == $blog_id) {
                continue;
            }

            if($excludedSites && in_array($blog_id, $excludedSites)) {
                continue;
            }

            switch_to_blog($blog_id);
            $this->post_copier->current_blog_id = $blog_id;
            $this->post_copier->copy_post($this->current_blog_id, $post->ID, false, true);
        }

        $sync_running = false;

        switch_to_blog($this->current_blog_id);
    }

    public function delete_post($post_ID)
    {
        $this->trash_or_delete($post_ID, false);
    }

    public function trash_post($post_ID)
    {
        $this->trash_or_delete($post_ID);
    }

    public function trash_or_delete($post_ID, $trash = true)
    {
        global $sync_running;

        if(!$this->get_option('syncEnable')) {
            return false;
        }

        if($sync_running) {
            return false;
        }

        if(isset($_POST['wordpress-multilingual-multisite-import'])) {
            return FALSE;
        }

        $masterSite = $this->get_option('masterSite');
        $syncOnlyMaster = $this->get_option('syncOnlyMaster');
        if(!$masterSite && $syncOnlyMaster) {
            wp_die( esc_html__('Sync only for master site enabled, but not master site is set', 'wordpress-multilingual-multisite') );
        } elseif($syncOnlyMaster && $masterSite && $masterSite != $this->current_blog_id){
            return false;
        }

        $post = get_post($post_ID);
        $post_type = $post->post_type;
        if(empty($post_type) || !$this->get_option('syncDelete' . $post_type)) {
            return false;
        }

        $this->translation_tables->set_table('posts_translations');
        $translated_posts = $this->translation_tables->get_translated_items_by_item_id($this->current_blog_id, $post_ID);
        if(empty($translated_posts)) {
            return false;
        }

        if($this->get_option('syncOnlySpecificPosts')) {
            $syncThisPost = get_post_meta($post_ID, 'wordpress_multilingual_multisite_sync_post', true);
            if(!$syncThisPost) {
                return false;
            }
        }

        $sync_running = 'trash';
        $excludedSites = $this->get_option('excludedSites');

        foreach ($translated_posts as $blog_id => $post_ID) {

            if(empty($post_id) || $blog_id == $this->current_blog_id || !is_numeric($blog_id)) {
                continue;
            }

            if($excludedSites && in_array($blog_id, $excludedSites)) {
                continue;
            }

            switch_to_blog($blog_id);

            if($trash) {
                wp_trash_post($post_ID);
            } else {
                wp_delete_post($post_ID);
            }

            restore_current_blog();
        }

        $sync_running = false;
    }

    public function sync_post_terms($post_ID, $post_after, $post_before)
    {
        global $sync_running;

        if(!$this->get_option('syncEnable')) {
            return false;
        }

        if($sync_running) {
            return false;
        }

        if(isset($_POST['wordpress-multilingual-multisite-import'])) {
            return FALSE;
        }

        $post_type = $post_after->post_type;
        if(empty($post_type) || !$this->get_option('syncEnable' . $post_type)) {
            return false;
        }
        
        $willPostBeCreate = array('auto-draft', 'draft');
        if($post_after->post_status == "publish" && in_array($post_before->post_status, $willPostBeCreate) && $this->get_option('syncCreate' . $post_type)) {
            return false;
        }

        $masterSite = $this->get_option('masterSite');
        $syncOnlyMaster = $this->get_option('syncOnlyMaster');
        if(!$masterSite && $syncOnlyMaster) {
            wp_die( esc_html__('Sync only for master site enabled, but not master site is set', 'wordpress-multilingual-multisite') );
        } elseif($syncOnlyMaster && $masterSite && $masterSite != $this->current_blog_id){
            return false;
        }

        if(!is_a($post_after, 'WP_Post')) {
            return false;
        }

        if (!current_user_can('edit_post', $post_after->ID)) {
            return $post_after->ID;
        }

        if ($post_type == 'revision') {
            return false;
        }

        if(empty($post_type) || !$this->get_option('syncTerms' . $post_type)) {
            return false;
        }

        if($this->get_option('syncOnlySpecificPosts')) {
            $syncThisPost = get_post_meta($post_ID, 'wordpress_multilingual_multisite_sync_post', true);
            if(!$syncThisPost) {
                return false;
            }
        }

        $original_post = $post_after;

        $original_taxonomy_terms = array();
        $original_taxonomies = get_object_taxonomies( $original_post ); //get_taxonomies();

        if(class_exists( 'woocommerce' ) && in_array($post_type, array('product', 'product_variation') ) ) {
            global $wpdb;
            $source_attribute_taxonomies = wc_get_attribute_taxonomies();
            $source_attribute_meta = get_post_meta($post_ID, '_product_attributes', true);
        }

        // uncategegorized
        $default_source_terms = array(
            get_option('default_category'),
            get_option('default_product_cat'),
        );

        foreach ($original_taxonomies as $original_taxonomy) {
            $original_taxonomy_terms[$original_taxonomy] = wp_get_post_terms($post_ID, $original_taxonomy, array("fields" => "all"));
        }

        $this->translation_tables->set_table('posts_translations');
        $translated_posts = $this->translation_tables->get_translated_items_by_item_id($this->current_blog_id, $post_ID);
        if(empty($translated_posts)) {
            return false;
        }

        // $sync_running = 'terms';

        remove_action( 'set_object_terms', array( 'WC_Post_Data', 'force_default_term' ), 10, 5 );

        $excludedSites = $this->get_option('excludedSites');

        foreach ($translated_posts as $blog_id => $post_id) {

            if(empty($post_id) || $blog_id == $this->current_blog_id || !is_numeric($blog_id)) {
                continue;
            }
        
            if($excludedSites && in_array($blog_id, $excludedSites)) {
                continue;
            }

            switch_to_blog($blog_id);
                
            // Update Terms
            $this->translation_tables->set_table('terms_translations');

            // add and register missing woo attributes
            if(class_exists( 'woocommerce' ) && in_array($post_type, array('product', 'product_variation')) && !empty($source_attribute_taxonomies)) {

                $target_attribute_taxonomies = wc_get_attribute_taxonomies();
                $not_existing_attributes = array();

                foreach ($source_attribute_taxonomies as $source_attribute_taxonomy) {
                    $source_attribute_taxonomy_name = 'pa_' . $source_attribute_taxonomy->attribute_name;
                    
                    foreach ($target_attribute_taxonomies as $target_attribute_taxonomy) {
                        $target_attribute_taxonomy_name = 'pa_' . $target_attribute_taxonomy->attribute_name;
                        if($source_attribute_taxonomy_name == $target_attribute_taxonomy_name) {
                            continue 2;
                        }
                    }

                    $not_existing_attributes[] = (array) $source_attribute_taxonomy;
                }                
            }

            if(class_exists( 'woocommerce' ) && !empty($not_existing_attributes)) {
                foreach ($not_existing_attributes as $not_existing_attribute) {
                    $wpdb->insert( $wpdb->prefix . 'woocommerce_attribute_taxonomies', $not_existing_attribute );

                    do_action( 'woocommerce_attribute_added', $wpdb->insert_id, $not_existing_attribute );

                    flush_rewrite_rules();
                    delete_transient( 'wc_attribute_taxonomies' );
                }
            }

            foreach ($original_taxonomy_terms as $taxonomy => $terms) {

                // empty existing terms
                wp_set_object_terms( $post_id, array(), $taxonomy);

                if(empty($terms)) {
                    continue;
                }

                if ( ! taxonomy_exists( $taxonomy ) ) {
                    register_taxonomy( $taxonomy, $original_post_type );
                }

                foreach ($terms as $term) {

                    if(in_array($term->term_id, $default_source_terms)) {
                        continue;
                    }

                    // Connect Term exists
                    $translated_terms = $this->translation_tables->get_translated_items_by_item_id($this->current_blog_id, $term->term_id);

                    if(isset($translated_terms[$this->current_blog_id]) && !empty($translated_terms[$blog_id])) {
                        $term_found = array( (int) $translated_terms[$blog_id]);
                        wp_set_object_terms( $post_id, $term_found, $taxonomy, true);
                        continue;
                    }
                    // Connected Term not exists -> check by name
                    if(term_exists( $term->name, $taxonomy)) {
                        wp_set_object_terms( $post_id, $term->name, $taxonomy, true);
                    // if not exists insert? 
                    } else {
                        // use term copier to keep translations ...
                        $new_term = $this->terms_copier->copy($this->current_blog_id, $term->term_id, $blog_id, false);
                        if(isset($new_term['term_id'])) {
                            $term_found = array( (int) $new_term['term_id']);
                            wp_set_object_terms( $post_id, $term_found, $taxonomy, true);
                        }
                    }

                }
            }

            // Regenerate WC lookup table
            if(in_array($post_type, array('product', 'product_variation'))) {

                if(!empty($source_attribute_meta )) {
                    update_post_meta($post_id, '_product_attributes', $source_attribute_meta);
                }

                $product = wc_get_product($post_id);
                if($product) {
                    $product->save();
                }
            }

            restore_current_blog();
        }

        flush_rewrite_rules();

        restore_current_blog();

        flush_rewrite_rules();
    }
}